(************************************************************)
(* This component is used for displaying extended hints.    *)
(* It uses a form with two labels to show the keyword and   *)
(* the description belonging to the keyword. The developer  *)
(* can specify an own procedure which provides the XHint-   *)
(* Window with necessarly data.                             *)
(*                                                          *)
(*                                                          *)
(* I needed this component for a program called SOP to      *)
(* show the user more small informations with hints in old  *)
(* fashiond format but a little bit more.                   *)
(*                                                          *)
(*                                                          *)
(* (c) January 2000 by Thomas Skarget                       *)
(* E-Mail: thomas.skarget@styria.com                        *)
(************************************************************)

unit ExHint;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, StdCtrls, Buttons;

type
  THintForm = class(TForm)
    Keyword: TLabel;
    Line: TShape;
    Text: TLabel;
    Closing: TPanel; // "Button" to close the window if you move with mouse over it.
    InActiveTimer: TTimer; // Timer which close the window if there was a mousemovement to an other control.
    HintTimer: TTimer; // Timer which close the window if there was no mousemovement.
    MoreButton: TSpeedButton;
    procedure FormShow(Sender: TObject);
    procedure ClosingClick(Sender: TObject);
    procedure InActiveTimerTimer(Sender: TObject);
    procedure MoreButtonClick(Sender: TObject);
    procedure HintTimerTimer(Sender: TObject);
    procedure FormHide(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
  private
    FDeltaX: Integer; // Value which specify the horizontal distance between mousecursor and hint-window's left border.
    FDeltaY: Integer; // Value which specify the vertical distance between mousecursor and hint-window's top border.
  public
    HintShowed: Boolean; // Decisionvariable for show window or not.
    HintStay: Boolean; // Decisionvariable for close window or not.
    procedure ShowExHint(Position: TPoint; Size: TSize; fKeyword: String; fText: TStringList); // Procedure with parameters, to show the window correct.
    procedure SetDisplayTime(Value: Cardinal);
    procedure SetDeltaX(Value: Cardinal);
    procedure SetDeltaY(Value: Cardinal);
  end;

  TNeedDataEvent = procedure (Key: String; Detail: TStringList) of Object;
  TMoreClickEvent = procedure (Sender: TObject; Key: String) of Object;

  TExHint = class(TComponent)
  private
    FDisplayTime: Cardinal;
    HintForm: THintForm;
    FKey: String;
    FDetail: TStringList;
    FMoreButtonHint: String;
    FCloseButtonHint: String;
    FKeyHint: String;
    FDetailHint: String;
    FShowHintOnHint: Boolean;
    FOnNeedData: TNeedDataEvent;
    FOnMoreClick: TMoreClickEvent;
    FOldAppHintPointer: TShowHintEvent;
    FOnClose: TNotifyEvent;
    FDeltaShowX: Integer;
    FDeltaShowY: Integer;
    FMoreButtonExist: Boolean;
    FTextColor: TColor;
    FBackgroundColor: TColor;
    procedure SetDisplayTime(Value: Cardinal);
    procedure SetKeyHint(Value: String);
    procedure SetDetailHint(Value: String);
    procedure SetMoreButtonHint(Value: String);
    procedure SetCloseButtonHint(Value: String);
    procedure SetShowHintOnHint(Value: Boolean);
    procedure SetDeltaShowX(Value: Integer);
    procedure SetDeltaShowY(Value: Integer);
    procedure SetMoreButtonExist(Value: Boolean);
    procedure SetTextColor(Value: TColor);
    procedure SetBackgroundColor(Value: TColor);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure AppShowHint(var HintStr: string; var CanShow: Boolean; var HintInfo: THintInfo);
    procedure NeedData;
    procedure MoreClick;
    procedure Close;
  published
    property DisplayTime: Cardinal read FDisplayTime write SetDisplayTime;
    property MoreButtonHint: String read FMoreButtonHint write SetMoreButtonHint;
    property KeyHint: String read FKeyHint write SetKeyHint;
    property DetailHint: String read FDetailHint write SetDetailHint;
    property CloseButtonHint: String read FCloseButtonHint write SetCloseButtonHint;
    property ShowHintOnHint: Boolean read FShowHintOnHint write SetShowHintOnHint;
    property DeltaShowX: Integer read FDeltaShowX write SetDeltaShowX;
    property DeltaShowY: Integer read FDeltaShowY write SetDeltaShowY;
    property MoreButtonExist: Boolean read FMoreButtonExist write SetMoreButtonExist;
    property BackgroundColor: TColor read FBackgroundColor write SetBackgroundColor;
    property TextColor: TColor read FTextColor write SetTextColor;
    property OnNeedData: TNeedDataEvent read FOnNeedData write FOnNeedData;
    property OnMoreClick: TMoreClickEvent read FOnMoreClick write FOnMoreClick;
    property OnClose: TNotifyEvent read FOnClose write FOnClose;
  end;



procedure Register;

implementation


{ THintForm }

{$R *.DFM}

procedure THintForm.FormShow(Sender: TObject);
begin
  HintTimer.Enabled := true;
  HintShowed := true;
end;

procedure THintForm.ClosingClick(Sender: TObject);
begin
  Hide;
end;

procedure THintForm.ShowExHint(Position: TPoint; Size: TSize; fKeyword: String; fText: TStringList);
begin
  Left := Position.x + Integer(FDeltaX);
  Top := Position.y + (FDeltaY);

  Keyword.Caption := fKeyword;
  Text.Caption := fText.Text;

// Set the size according to the height and witdth of the text-body.
// Arrange the more- and the closing-button according to the text.

  if (Text.Width > Keyword.Width) then
  begin
    Width := 8 + Text.Width + 17 + 8;
    Height := 8 + 24 + Text.Height;
    Line.Width := Text.Width;
    Closing.Left := Line.Left + Line.Width + 4;
    MoreButton.Left := Text.Left + Text.Width + 4;
    MoreButton.Top := Line.Top + Text.Height div 2 - 8;
  end
  else
  begin
    Width := 8 + Keyword.Width + 17 + 8;
    Height := 8 + 24 + Text.Height;
    Line.Width := Keyword.Width;
    Closing.Left := Line.Left + Line.Width + 4;
    MoreButton.Left := Text.Left + Keyword.Width + 4;
    MoreButton.Top := Line.Top + Text.Height div 2 - 8;
  end;

// Check if the window is able to be showed at top-right position.
// If not change showing position.

  if (Left + Width > Screen.Width) then
    Left := Left - Width - 2 * Integer(FDeltaX);
  if (Top + Height > Screen.Height) then
    Top := Top - Height - 2 * Integer(FDeltaY);

//  Display the hint-window.

  Show;
end;


procedure THintForm.InActiveTimerTimer(Sender: TObject);
begin
  if (not Active) then
    Hide;
end;

procedure THintForm.MoreButtonClick(Sender: TObject);
begin
// Fire owner's more-button click event.
  (Self.Owner as TExHint).MoreClick;
end;

procedure THintForm.HintTimerTimer(Sender: TObject);
begin
  Hide;
end;

procedure THintForm.FormHide(Sender: TObject);
begin
// Set variables to the standard values.
  HintShowed := false;
  HintTimer.Enabled := false;
  HintStay := false;
// Fire owner's close event.
  (Owner as TExHint).Close;
end;

procedure THintForm.FormCreate(Sender: TObject);
begin
  HintShowed := false;
  HintStay := false;
  HintTimer.Enabled := false;
end;

procedure THintForm.SetDisplayTime(Value: Cardinal);
begin
  HintTimer.Interval := Value;
end;

procedure THintForm.SetDeltaX(Value: Cardinal);
begin
  FDeltaX := Value;
end;

procedure THintForm.SetDeltaY(Value: Cardinal);
begin
  FDeltaY := Value;
end;

procedure THintForm.FormMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
begin
  HintStay := true;
  HintTimer.Enabled := false;
  InActiveTimer.Enabled := false;
end;


{ TExHint }

constructor TExHint.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  HintForm := THintForm.Create(self);
  FKey := '';
  FDetail := TStringList.Create;
// Mind the original windows hint procedure
  FOldAppHintPointer := Application.OnShowHint;
  FDisplayTime := 1;
  FCloseButtonHint := '';
  FMoreButtonHint := '';
  FKeyHint := '';
  FDetailHint := '';
  FShowHintOnHint := false;
  FDeltaShowX := 0;
  FDeltaShowY := 20;
  FTextColor := clBlack;
  FBackgroundColor := clInfoBk;
// Overwrite the windows (application) hint procedure
  Application.OnShowHint := AppShowHint;
end;

destructor TExHint.Destroy;
begin
  HintForm.Free;
  FDetail.Free;
// Restore the windows hint-procedure.
  Application.OnShowHint := FOldAppHintPointer;
  inherited Destroy;
end;

procedure TExHint.SetDisplayTime(Value: Cardinal);
begin
  FDisplayTime := Value;
// Check if HintForm's controls are not nil. Necessary for design-time.
  if (Assigned(HintForm.HintTimer)) then
    HintForm.SetDisplayTime(FDisplayTime);
end;

procedure TExHint.SetKeyHint(Value: String);
begin
  FKeyHint := Value;
// Check if HintForm's controls are not nil. Necessary for design-time.
  if (Assigned(HintForm.Keyword)) then
    HintForm.Keyword.Hint := FKeyHint;
end;

procedure TExHint.SetDetailHint(Value: String);
begin
  FDetailHint := Value;
// Check if HintForm's controls are not nil. Necessary for design-time.
  if (Assigned(HintForm.Text)) then
    HintForm.Text.Hint := FDetailHint;
end;

procedure TExHint.SetMoreButtonHint(Value: String);
begin
  FMoreButtonHint := Value;
// Check if HintForm's controls are not nil. Necessary for design-time.
  if (Assigned(HintForm.MoreButton)) then
    HintForm.MoreButton.Hint := FMoreButtonHint;
end;

procedure TExHint.SetCloseButtonHint(Value: String);
begin
  FCloseButtonHint := Value;
// Check if HintForm's controls are not nil. Necessary for design-time.
  if (Assigned(HintForm.Closing)) then
    HintForm.Closing.Hint := FCloseButtonHint;
end;

procedure TExHint.SetShowHintOnHint(Value: Boolean);
begin
  FShowHintOnHint := Value;
  HintForm.ShowHint := FShowHintOnHint;
end;

procedure TExHint.SetMoreButtonExist(Value: Boolean);
begin
  FMoreButtonExist := Value;
// Check if HintForm's controls are not nil. Necessary for design-time.
  if (Assigned(HintForm.MoreButton)) then
    HintForm.MoreButton.Visible := FMoreButtonExist;
end;

procedure TExHint.AppShowHint(var HintStr: string; var CanShow: Boolean; var HintInfo: THintInfo);
var
  pos: TPoint;
  size: TSize;

const
  oldPos: TPoint = ();

begin
  GetCursorPos(HintInfo.CursorPos);
  pos := HintInfo.CursorPos;

  FKey := HintStr;
  FDetail.Clear;

// Fire needdata-event to get the data for displaying
  NeedData;

// If the user moved the mouse over the hint-window, the hint-window is showed
// until the close-button is clicked. Do not readraw the hint-window.
  if (not HintForm.HintStay) then
  begin
    if ((oldPos.x <> pos.x) and (oldPos.y <> pos.y)) then
    begin
      oldPos.x := pos.x;
      oldPos.y := pos.y;

      if ((FKey <> '') and (not HintForm.HintShowed) and (FDetail.Text <> '')) then
      begin
// Show the hint-window on mousecursor position.
        HintForm.ShowExHint(pos,size,FKey,FDetail);
      end
      else
      begin
        HintForm.Hide;
      end;
    end;
  end;

// If no detailed text is present use the original window-hint.
  if (FDetail.Text = '') then
    CanShow := true
  else
    CanShow := false;
end;

procedure TExHint.NeedData;
begin
  if (Assigned(FOnNeedData)) then
    FOnNeedData(FKey,FDetail);
end;

procedure TExHint.MoreClick;
begin
  if (Assigned(FOnMoreClick)) then
    FOnMoreClick(HintForm.MoreButton,HintForm.Keyword.Caption);
end;

procedure TExHint.Close;
begin
  if (Assigned(FOnClose)) then
    FOnClose(self);
end;

procedure TExHint.SetDeltaShowX(Value: Integer);
begin
  FDeltaShowX := Value;
  HintForm.SetDeltaX(FDeltaShowX);
end;

procedure TExHint.SetDeltaShowY(Value: Integer);
begin
  FDeltaShowY := Value;
  HintForm.SetDeltaY(FDeltaShowY);
end;

procedure TExHint.SetTextColor(Value: TColor);
begin
  FTextColor := Value;
  with HintForm do
  begin
// Check if HintForm's controls are not nil. Necessary for design-time.
    if (Assigned(Keyword)) then
    begin
      Keyword.Font.Color := FTextColor;
      Line.Brush.Color := FTextColor;
      Closing.Font.Color := FTextColor;
      MoreButton.Font.Color := FTextColor;
      Text.Font.Color := FTextColor;
    end;
  end;
end;

procedure TExHint.SetBackgroundColor(Value: TColor);
begin
  FBackgroundColor := Value;
  with HintForm do
  begin
// Check if HintForm's controls are not nil. Necessary for design-time.
    if (Assigned(Keyword)) then
    begin
      Color := FBackgroundColor;
      Closing.Color := FBackgroundColor;
    end;
  end;
end;



procedure Register;
begin
  RegisterComponents('skt', [TExHint]);
end;

end.
